@@ -958,6 +958,21 @@ void BayonneSession::setConnecting(const char *evname)
 	setRunning();
 }
 
+bool BayonneSession::setReconnect(const char *enc, timeout_t framing)
+{
+	Event ev;
+
+	memset(&ev, 0, sizeof(ev));
+	ev.id = ENTER_RECONNECT;
+	ev.reconnect.encoding = enc;
+	ev.reconnect.framing = framing;
+	if(!enterReconnect(&ev))
+		return false;
+
+	setState(STATE_RECONNECT);
+	return true;
+}
+
 void BayonneSession::setRunning(void)
 {
 	setState(STATE_RUNNING);
