@@ -1263,6 +1263,7 @@ bool BayonneSession::stateJoin(Event *event)
 	const char *err;
         event_t pid = PART_DISCONNECT;
         bool rtn;  
+	BayonneSession *p;
 
 	if(enterJoin(event))
 		return true;
@@ -1281,6 +1282,7 @@ bool BayonneSession::stateJoin(Event *event)
 		if(state.join.dtmf && !dtmf)
 			dtmf = enableDTMF();
                 setSymbol("script.error", "none");
+		time(&state.join.start);
                 return true; 
 	case AUDIO_DISCONNECT:
 		memcpy(&hold_event, event, sizeof(hold_event));
@@ -1327,9 +1329,12 @@ bool BayonneSession::stateJoin(Event *event)
 		goto part;
 	}
 
+	p = getSid(var_pid);
 	rtn = enterCommon(event);
 	if(!isJoined())
 	{
+		if(p)
+			BayonneBinder::reportBilling(p, this, state.join.start);
 		part(pid);
 		event->id = EXIT_PARTING;
 		enterJoin(event);
@@ -1337,6 +1342,9 @@ bool BayonneSession::stateJoin(Event *event)
 	return rtn;
 
 part:
+	p = getSid(var_pid);
+	if(p)
+			BayonneBinder::reportBilling(p, this, state.join.start);
 	if(scriptEvent(msg))
 		state.join.hangup = false;
 	else if(signalScript(SIGNAL_PART))
