@@ -336,7 +336,7 @@ bool Driver::isReachable(const char *proxy)
 
 	if(strchr(proxy, '@'))
 		proxy = strchr(proxy, '@') + 1;
-	else if(!strnicmp(proxy, "sip:4", 4))
+	else if(!strnicmp(proxy, "sip:", 4))
 		proxy += 4;
 
 	img = useImage();
@@ -398,8 +398,6 @@ bool Driver::isExternal(const char *ext)
 	return rtn;
 }
 
-
-
 bool Driver::isRegistered(const char *ext)
 {
 	char buf[65];
@@ -435,11 +433,12 @@ bool Driver::isRegistered(const char *ext)
 	}
 	if(reg && !strnicmp(reg->type, "ext", 3))
 		rtn = reg->isActive();
+	else if(reg && !stricmp(reg->type, "gw"))
+		rtn = reg->isActive();
 	endImage(img);
 	return rtn;
 }
 
-
 bool Driver::isAvailable(const char *ext)
 {
 	char buf[65];
@@ -447,6 +446,7 @@ bool Driver::isAvailable(const char *ext)
 	Registry *reg;
 	bool rtn = false;
 	Name *scr;
+	bool regext = false, gw = false;
 
 	if(!ext)
 		return false;
@@ -473,39 +473,55 @@ bool Driver::isAvailable(const char *ext)
 			reg = (Registry *)img->getPointer(buf);
 		}
 	}
-	if(reg && !strnicmp(reg->type, "ext", 3) && reg->isActive())
+	if(reg && !strnicmp(reg->type, "ext", 3))
+		regext = true;
+	else if(reg && !stricmp(reg->type, "gw"))
+		gw = true;
+	if((regext || gw) && reg->isActive())
 	{
 		if(!reg->call_limit)
 			rtn = true;
 		else if(reg->call_limit > reg->traffic->active_calls)
 			rtn = true;
-		switch(reg->traffic->presence)
-		{
-		case SIPTraffic::P_BUSY:
-		case SIPTraffic::P_AWAY:
-		case SIPTraffic::P_DND:
+		if(!gw)
 			rtn = false;
-		default:
-			break;
-		}
+		else
+			switch(reg->traffic->presence)
+			{
+			case SIPTraffic::P_BUSY:
+			case SIPTraffic::P_AWAY:
+			case SIPTraffic::P_DND:
+				rtn = false;
+			default:
+				break;
+			}
 	}
 	endImage(img);
 	return rtn;
 }
 
-
-
 unsigned Driver::getRegistration(regauth_t *data, unsigned count)
 {
 	unsigned idx = 0;
 	ScriptImage *img = useImage();
 	Registry *reg = (Registry *)img->getPointer("_sip_registry_");
+	bool ext, gw, anon;
 
 	while(idx < count && reg)
 	{
-		if(!stricmp(reg->type, "anon"))
+		anon = false;
+		ext = false;
+		gw = false;
+		if(!strnicmp(reg->type, "ext", 3))
+			ext = true;
+		else if(!stricmp(reg->type, "gw"))
+			gw = true;
+		else if(!stricmp(reg->type, "anon"))
+			anon = true;
+
+		if(anon)
 			data[idx].remote = "*";
-		else if(!strnicmp(reg->type, "ext", 3) && !reg->traffic->active)
+		else if((ext || gw) && !reg->traffic->active)
 			data[idx].remote = "-";
 		else if(!strnicmp(reg->proxy, "sip:", 4))
 			data[idx].remote = reg->proxy + 4;
@@ -513,11 +529,11 @@ unsigned Driver::getRegistration(regauth_t *data, unsigned count)
 			data[idx].remote = reg->proxy;
 		data[idx].userid = reg->userid;
 		data[idx].type = reg->type;
-		if(!stricmp(reg->type, "anon"))
+		if(anon)
 			data[idx].status = "active";
-		else if(!strnicmp(reg->type, "ext", 3) && !reg->traffic->active)
+		else if((ext || gw) && !reg->traffic->active)
 			data[idx].status = "offline";
-		else if(!strnicmp(reg->type, "ext", 3) && reg->traffic->getTimer() > 0)
+		else if(ext && reg->traffic->getTimer() > 0)
 		{
 			switch(reg->traffic->presence)
 			{
@@ -537,6 +553,13 @@ unsigned Driver::getRegistration(regauth_t *data, unsigned count)
 					data[idx].status = "busy";
 			}
 		}
+		else if(gw && reg->traffic->getTimer() > 0)
+		{
+			if(!reg->call_limit || reg->call_limit > reg->traffic->active_calls)
+				data[idx].status = "online";
+			else
+				data[idx].status = "busy";
+		}
 		else if(!reg->proxy || !reg->userid)
 			data[idx].status = "invalid";
 		else if(!reg->traffic->active)
@@ -847,6 +870,7 @@ void Driver::stopDriver(void)
 	unsigned c = count;
 	timeslot_t ts = timeslot;
 	BayonneSession *s;
+	bool extgw;
 
 	if(running)
 	{
@@ -857,7 +881,14 @@ void Driver::stopDriver(void)
 			reg = (Registry *)img->getPointer("_sip_registry_");
 			while(reg)
 			{
-				if(strnicmp(reg->type, "ext", 3) || stricmp(reg->type, "peer"))
+				extgw = false;
+				if(!strnicmp(reg->type, "ext", 3))
+					extgw = true;
+				else if(!stricmp(reg->type, "gw"))
+					extgw = true;
+				else if(!stricmp(reg->type, "peer"))
+					extgw = true;
+				if(!extgw)
 					updateExpires(reg, 0);
 				reg = reg->next;
 			}
@@ -1141,6 +1172,7 @@ void Driver::registerRequest(eXosip_event_t *sevent)
 	int pos = 0;
 	char *cp;
 	InetAddress addr;
+	const char *extgw = NULL;
 
 	if(osip_message_get_authorization(sevent->request, 0, &auth) != 0)
 		goto challenge;
@@ -1157,12 +1189,19 @@ void Driver::registerRequest(eXosip_event_t *sevent)
 	server->enter();
 	img = server->getActive();
 	reg = (Registry *)img->getPointer(buf);	
-	if(!reg || strnicmp(reg->type, "ext", 3))
+	if(reg && !strnicmp(reg->type, "ext", 3))
+		extgw = "external";
+	else if(reg && !stricmp(reg->type, "gw"))
+		extgw = "gateway";
+	else if(reg && !stricmp(reg->type, "peer"))
+		extgw = "peer";
+
+	if(!reg || extgw == NULL)
 	{
 fail401:
 
 		server->leave();
-		slog.debug("rejecting external %s", auth->username); 
+		slog.debug("rejecting invalid %s", auth->username); 
 		eXosip_lock();
 		eXosip_message_build_answer(sevent->tid, 401, &reply);
 		eXosip_message_send_answer(sevent->tid, 401, reply);
@@ -1233,7 +1272,8 @@ sip200:
 
 		reg->traffic->setTimer(pos * 1000);
 		reg->traffic->active = true;
-		slog.debug("registered external %s from %s; expires=%d", auth->username, buf, pos);
+		slog.debug("registered %s %s from %s; expires=%d", 
+			extgw, auth->username, buf, pos);
 	}
 	else
 	{
@@ -1241,7 +1281,8 @@ sip200:
 		reg->traffic->active = false;
 		reg->traffic->setTimer(0);
 		reg->traffic->presence = SIPTraffic::P_AVAILABLE;
-		slog.debug("unregister external %s from %s", auth->username, buf);
+		slog.debug("unregister %s %s from %s", 
+			extgw, auth->username, buf);
 	}
 answer200:
 	server->leave();
@@ -1266,35 +1307,39 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 {
 	Name *scr = img->getCurrent();
 	Registry *reg = (Registry *)img->getMemory(sizeof(Registry));
+	Registry *preg;
         const char *dur = ScriptChecks::findKeyword(line, "timeout");
 	const char *insecure = ScriptChecks::findKeyword(line, "insecure");
 	const char *lcp, *cp;
 	char buf[1024];
 	char nbuf[32];
 	unsigned len, pos;
-	char *p;
+	char *p, *sp, *tok;
 	char *uri;
 	long mult = 1000;
 	osip_message_t *msg = NULL; 
 	InetAddress host;
-	bool ext = false;
+	bool ext = false, gw = false;
 	Image *ip = (Image *)img;
 	Keydata *cfg = NULL;
+	const char *gateways = NULL;
 
 	memset(reg, 0, sizeof(Registry));
 
 	if(!strnicmp(line->cmd, "ext", 3))
 		ext = true;
+	else if(!stricmp(line->cmd, "gateway"))
+		gw = true;
 
-	if(ext)
+	if(ext || gw)
 	{
 		if(!ScriptChecks::useKeywords(line, 
-			"=reconnect=peering=secret=encoding=framing=dtmf=user=count=device=info"))
+			"=reconnect=peering=secret=encoding=framing=dtmf=user=count=device=info=answer"))
 				return "invalid keywords used for sip external";
 	}
 	else if(!ScriptChecks::useKeywords(line, 
 		"=uri=timeout=server=proxy=userid=secret=type=realm=dtmf=reconnect"
-		"=limit=public=encoding=framing=local=caller=service=peering"))
+		"=limit=public=encoding=framing=local=caller=service=peering=answer=gateways"))
 			return "invalid keywords for sip registry";
 
 	reg->protocol = "sip";
@@ -1311,10 +1356,14 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	reg->encoding = ScriptChecks::findKeyword(line, "encoding");
 	reg->caller = ScriptChecks::findKeyword(line, "caller");
 	reg->reconnect = ScriptChecks::findKeyword(line, "reconnect");
+	reg->answer = ScriptChecks::findKeyword(line, "answer");
+	gateways = ScriptChecks::findKeyword(line, "gateways");
 
-	if(ext)
+	if(ext || gw)
 		reg->localid = ScriptChecks::findKeyword(line, "user");
-	else
+	if(gw && !reg->localid)
+		reg->localid = ScriptChecks::findKeyword(line, "userid");
+	if(!ext)
 	{
 		reg->service = ScriptChecks::findKeyword(line, "service");
 		reg->localid = ScriptChecks::findKeyword(line, "local");
@@ -1322,6 +1371,18 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 			reg->localid = reg->service;
 	}
 
+	if(!gw && reg->type && !stricmp(reg->type, "gateway"))
+		gw = true;
+
+	if(!gw && reg->type && !stricmp(reg->type, "gw"))
+		gw = true;
+
+	if(gw)
+	{
+		gateways = NULL;
+		reg->type = "gw";
+	}
+
 	if(!reg->service)
 	{
 		reg->service = scr->name;
@@ -1390,6 +1451,9 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 			if(cp)
 				reg->framing = atol(cp);
 		}
+		cp = cfg->getLast("answer");
+		if(cp && !reg->answer)
+			reg->answer = cp;
 		cp = cfg->getLast("reconnect");
 		if(cp && !reg->reconnect)
 			reg->reconnect = ip->dupString(cp);
@@ -1435,7 +1499,7 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	{
 		if(!reg->type && ext)
 			reg->type = "extern";
-		else
+		else if(!gw)
 		{
 			reg->hostid = getLast("proxy");
 			if(!reg->hostid || !*reg->hostid)
@@ -1585,6 +1649,14 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 		slog.debug("register %s externally", reg->localid);
 		goto add;
 	}
+	else if(gw)
+	{
+		slog.debug("register %s gateway", reg->localid);
+		snprintf(buf, sizeof(buf), "sip.%s", reg->localid);
+		setPointer(buf, reg);
+		reg->proxy = NULL;
+		goto add;
+	}
 
 	slog.debug("register %s as %s on %s", reg->localid, reg->userid, reg->proxy);
 
@@ -1641,13 +1713,13 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	else
 		p = "";
 
-	snprintf(nbuf, sizeof(nbuf), "sip.%s%s",
-		inet_ntoa(host.getAddress()), p);
-	img->setPointer(nbuf, reg);
-
 	snprintf(buf, sizeof(buf), "sipreg.%d", reg->regid);
 	img->setPointer(buf, reg);
 
+        snprintf(nbuf, sizeof(nbuf), "sip.%s%s",
+                inet_ntoa(host.getAddress()), p);
+        img->setPointer(nbuf, reg);
+
 add:
 	snprintf(buf, sizeof(buf), "uri.%s", reg->localid);
 	reg->traffic = Driver::sip.getTraffic(buf);
@@ -1664,6 +1736,44 @@ add:
 		img->setPointer(buf, &Driver::sip);
 	}
 
+	if(gateways)
+	{
+		setString(buf + 5, sizeof(buf) - 5, gateways);
+		gateways = strtok_r(buf, ",;\t\r\n", &tok);
+	}
+	while(gateways)
+	{
+		p = strrchr(gateways, ':');
+		if(p && !stricmp(p, ":5060"))
+			*p = 0;
+
+		sp = (char *)(buf - 4);
+		sp[0] = 's';
+		sp[1] = 'i';
+		sp[2] = 'p';
+		sp[3] = '.';
+		if(img->getPointer(sp))
+			goto ngw;
+
+		preg = (Registry *)img->getMemory(sizeof(Registry));
+		memcpy(preg, reg, sizeof(Registry));
+		preg->proxy = ip->dupString(gateways);
+		setPointer(sp, preg);
+		
+		host = gateways;
+		if(p)
+			*p = ':';
+		else
+			p = "";
+
+		snprintf(nbuf, sizeof(nbuf), "sip.%s%s",
+			inet_ntoa(host.getAddress()), p);
+		img->setPointer(nbuf, reg);
+		
+ngw:
+		gateways = strtok_r(NULL, ",;\t\r\n", &tok);
+	}
+
 	return "";
 }
 
@@ -1834,14 +1944,17 @@ void Driver::run(void)
 		case EXOSIP_CALL_MESSAGE_NEW:
 			if(!sevent->request)
 				break;
-			if(!MSG_IS_INFO(sevent->request))
-				break;
 			session = getCall(sevent->cid);
 			if(!session)
 			{
 				slog.warn("sip: got info for non-existant call");
 				goto killit;
 			}
+			if(!MSG_IS_INFO(sevent->request))
+			{
+				slog.debug("%s: in dialog %s currently unsupported", session->logname, sevent->request->sip_method);
+				break;
+			}
 			ct = sevent->request->content_type;
 			if(!ct || !ct->type || !ct->subtype)
 				break;
@@ -2158,6 +2271,8 @@ killit:
 				publishRequest(sevent, to->url->username);
 				goto cleanup;
 			}
+			else
+				slog.debug("sip: out of dialog msg %s", sevent->request->sip_method);
 			break;
 		case EXOSIP_CALL_REINVITE:
                         if(!sevent->request)
@@ -2460,6 +2575,11 @@ killit:
 					auth = "peer";
 					goto validate;
 				}
+				if(!stricmp(reg->type, "gw"))
+				{
+					auth = "peer";
+					goto validate;
+				}
 				goto sip404;
 			}
 
@@ -2470,6 +2590,8 @@ killit:
 				auth = reg->type;
 			if(!strnicmp(auth, "ext", 3))
 				auth = "peer";
+			if(!stricmp(auth, "gw"))
+				auth = "peer";
 			if(!stricmp(auth, "friend") && peer)
 				auth = "peer";
 			else if(!stricmp(auth, "friend") && proxy)
@@ -2620,7 +2742,7 @@ sip480:
 			{
 				++reg->traffic->call_attempts.iCount;
 				++reg->traffic->call_complete.iCount;
-				if(!stricmp(reg->type, "anon") || !strnicmp(reg->type, "ext", 3))
+				if(!stricmp(reg->type, "anon") || !strnicmp(reg->type, "ext", 3) || !stricmp(reg->type, "gw"))
 					snprintf(cbuf, sizeof(cbuf), "uri.%s", reg->localid);
 				else
 					snprintf(cbuf, sizeof(cbuf), "sipreg.%d", reg->regid);
@@ -2842,10 +2964,20 @@ sip415:
                         if(!s->postEvent(&event))
                                 goto sip404;
 
-			eXosip_lock();
-			eXosip_call_send_answer(sevent->tid, 180, NULL);
-			eXosip_unlock();
-			Thread::yield();
+
+			cp = NULL;
+			if(reg)
+				cp = reg->answer;
+			if(!cp)
+				cp = "200";
+
+			if(strstr(cp, "180") || !stricmp(cp, "no"))
+			{
+				eXosip_lock();
+				eXosip_call_send_answer(sevent->tid, 180, NULL);	
+				eXosip_unlock();
+				Thread::yield();
+			}
 
 			eXosip_lock();
 			eXosip_call_build_answer(sevent->tid, 200, &msg);
