@@ -762,6 +762,7 @@ dialer:
 			if(!strnicmp(cp, "sip:", 4))
 				cp += 4;
 			snprintf(buf, sizeof(buf), "sip:%s", cp);	
+			sp = strrchr(buf, ':');
 		}
 		if(sp && stricmp(sp, ":5060"))
 			sp = NULL;
@@ -816,7 +817,7 @@ dialing:
 
 			if(!strnicmp(reg->type, "ext", 3))
 				setConst("session.identity", reg->localid);
-			else
+			else if(reg->service)
 				setConst("session.service", reg->service);
 
 			snprintf(nbuf, sizeof(nbuf), "%d", instance);
