@@ -15,7 +15,9 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include "module.h"
+#ifndef	WIN32
 #include "private.h"
+#endif
 
 namespace moduleWebservice {
 using namespace ost;
@@ -162,6 +164,7 @@ void Session::xmlStart(void)
 	if(!s)
 		xmlFault(XML_COMMAND_FAILED, "start");
 
+	s->setConst("session.manager", "webservice");
 	sid = s->getExternal("session.id");
 	xmlHeader("start");
 	snprintf(buffer, sizeof(buffer),
