@@ -91,6 +91,10 @@ ScriptRuntime()
 			(Check)&Checks::chkSleep},
 		{"route", false, (Method)&Methods::scrRoute,
 			(Check)&Checks::chkRoute},
+		{"droute", true, (Method)&Methods::scrRoute,
+			(Check)&Checks::chkRoute},
+		{"sroute", true, (Method)&Methods::scrSRoute,
+			(Check)&Checks::chkRoute},
 #ifdef  WIN32
                 {"link", false, (Method)&Methods::scrCopy,
                         (Check)&Checks::chkMove},
@@ -536,6 +540,7 @@ bool Runtime::command(char *cmd)
 		s = startDialing(dial, cmd, caller, display);
 		if(!s)
 			return false;
+		s->setConst("session.manager", "server");
 		s->leave();
 		return true;
 	}	
